﻿using System;
using System.IO;
using Vintagestory.API.Common;
using Vintagestory.API.Common.Entities;
using Vintagestory.API.MathTools;
using Vintagestory.API.Server;

//Note from Playah: Some of the following code has been copied and haphazardly modified from
//https://github.com/anegostudios/vssurvivalmod/blob/622cf16701b220ab55e54bd0c74168052d155960/Entities/EntityThrownStone.cs (v1.15.2)

namespace snowballs
{
    public class EntityThrownSnowball : Entity
    {
        protected bool beforeCollided;
        protected bool stuck;

        protected long msLaunch;
        protected Vec3d motionBeforeCollide = new Vec3d();

        public Entity FiredBy;
        public float Damage;
        public ItemStack ProjectileStack;

        public SimpleParticleProperties SnowballTrail = new SimpleParticleProperties();
        public SimpleParticleProperties SnowballImpactBits = new SimpleParticleProperties();
        public SimpleParticleProperties SnowballImpactPuff = new SimpleParticleProperties();
        public Vec3f SnowballTrailSpread;
        public Vec3f SnowballImpactBitsSpread;
        public Vec3f SnowballImpactPuffSpread;
        public int RandomGray;

        public override bool IsInteractable
        {
            get { return false; }
        }

        public override void Initialize(EntityProperties properties, ICoreAPI api, long InChunkIndex3d)
        {
            base.Initialize(properties, api, InChunkIndex3d);

            msLaunch = World.ElapsedMilliseconds;

            if (ProjectileStack.Collectible != null)
            {
                ProjectileStack.ResolveBlockOrItem(World);
            }

            //Default particles
            SnowballTrail.MinQuantity = 1;
            SnowballTrail.AddQuantity = 1;
            SnowballTrail.AddPos = new Vec3d(0.25, 0.25, 0.25);
            SnowballTrail.LifeLength = 0.3f;
            SnowballTrail.addLifeLength = 0.5f;
            SnowballTrail.MinSize = 0.3f;
            SnowballTrail.MaxSize = 1.0f;
            SnowballTrail.SizeEvolve = new EvolvingNatFloat(EnumTransformFunction.LINEAR, -SnowballTrail.Size);
            SnowballTrail.GravityEffect = 0.5f;
            SnowballTrail.ParticleModel = EnumParticleModel.Cube;
            SnowballTrailSpread = new Vec3f(1f, 1f, 1f);

            SnowballImpactBits.MinQuantity = 15;
            SnowballImpactBits.AddQuantity = 5;
            SnowballImpactBits.AddPos = new Vec3d(0.25, 0.25, 0.25);
            SnowballImpactBits.LifeLength = 1f;
            SnowballImpactBits.addLifeLength = 1f;
            SnowballImpactBits.MinSize = 1.5f;
            SnowballImpactBits.MaxSize = 2.0f;
            SnowballImpactBits.GravityEffect = 2.0f;
            SnowballImpactBits.ParticleModel = EnumParticleModel.Cube;
            SnowballImpactBitsSpread = new Vec3f(10f, 10f, 10f);

            SnowballImpactPuff.MinQuantity = 15;
            SnowballImpactPuff.AddQuantity = 5;
            SnowballImpactPuff.AddPos = new Vec3d(0.25, 0.25, 0.25);
            SnowballImpactPuff.LifeLength = 0.5f;
            SnowballImpactPuff.addLifeLength = 1f;
            SnowballImpactPuff.MinSize = 0.5f;
            SnowballImpactPuff.MaxSize = 1f;
            SnowballImpactPuff.SizeEvolve = new EvolvingNatFloat(EnumTransformFunction.LINEAR, SnowballImpactPuff.Size * 2f);
            SnowballImpactPuff.GravityEffect = 0.05f;
            SnowballImpactPuff.ParticleModel = EnumParticleModel.Quad;
            //These don't seem to do anything yet.
            //SnowballImpactPuff.WindAffected = true;
            //SnowballImpactPuff.WindAffectednes = 1;
            SnowballImpactPuffSpread = new Vec3f(2f, 2f, 2f);
        }

        public override void OnGameTick(float dt)
        {
            base.OnGameTick(dt);
            if (ShouldDespawn) return;

            EntityPos pos = SidedPos;

            stuck = Collided;

            
            pos.Pitch = (World.ElapsedMilliseconds / 100f) % GameMath.TWOPI;
            pos.Roll = 0;
            pos.Yaw = (World.ElapsedMilliseconds / 150f) % GameMath.TWOPI;

            //Particle trail
            SnowballTrail.MinPos = pos.XYZ;

            RandomGray = new Random().Next(200, 255);
            SnowballTrail.Color = ColorUtil.ColorFromRgba(255, RandomGray, RandomGray, 255);
            
            SnowballTrail.MinVelocity = (pos.Motion.ToVec3f() * 12) + (SnowballTrailSpread * -0.5f);
            SnowballTrail.AddVelocity = SnowballTrailSpread;

            World.SpawnParticles(SnowballTrail);


            if (stuck)
            {
                if (!beforeCollided && World is IServerWorldAccessor)
                {
                    float strength = GameMath.Clamp((float)motionBeforeCollide.Length() * 4, 0, 1);
                    World.PlaySoundAt(new AssetLocation("sounds/thud"), this, null, false, 32, strength);

                    BreakSnowball();

                    WatchedAttributes.MarkAllDirty();
                }
                beforeCollided = true;
                return;
            }


            if (World is IServerWorldAccessor)
            {
                Entity entity = World.GetNearestEntity(ServerPos.XYZ, 5f, 5f, (e) => {
                    if (e.EntityId == this.EntityId || (FiredBy != null && e.EntityId == FiredBy.EntityId && World.ElapsedMilliseconds - msLaunch < 500) || !e.IsInteractable)
                    {
                        return false;
                    }

                    double dist = e.CollisionBox.ToDouble().Translate(e.ServerPos.X, e.ServerPos.Y, e.ServerPos.Z).ShortestDistanceFrom(ServerPos.X, ServerPos.Y, ServerPos.Z);
                    return dist < 0.5f;
                });

                if (entity != null)
                {
                    bool didDamage = entity.ReceiveDamage(new DamageSource() { Source = EnumDamageSource.Entity, SourceEntity = FiredBy == null ? this : FiredBy, Type = EnumDamageType.BluntAttack }, Damage);
                    World.PlaySoundAt(new AssetLocation("sounds/thud"), this, null, false, 32);

                    if (FiredBy is EntityPlayer && didDamage)
                    {
                        World.PlaySoundFor(new AssetLocation("sounds/player/projectilehit"), (FiredBy as EntityPlayer).Player, false, 24);
                    }

                    BreakSnowball();
                }
            }

            beforeCollided = false;
            motionBeforeCollide.Set(pos.Motion.X, pos.Motion.Y, pos.Motion.Z);

            SetRotation();
        }

        public virtual void BreakSnowball()
        {
            EntityPos pos = SidedPos;

            SnowballImpactBits.MinPos = pos.XYZ;

            RandomGray = new Random().Next(200, 255);
            SnowballImpactBits.Color = ColorUtil.ColorFromRgba(255, RandomGray, RandomGray, 255);

            SnowballImpactBits.MinVelocity = (pos.Motion.ToVec3f() * 20) + (SnowballImpactBitsSpread * -0.5f);
            SnowballImpactBits.AddVelocity = SnowballImpactBitsSpread;
            World.SpawnParticles(SnowballImpactBits);

            SnowballImpactPuff.MinPos = pos.XYZ;

            SnowballImpactPuff.Color = ColorUtil.ColorFromRgba(255, 225, 215, 50);
            SnowballImpactPuff.OpacityEvolve = new EvolvingNatFloat(EnumTransformFunction.LINEAR, -50);

            SnowballImpactPuff.MinVelocity = SnowballImpactPuffSpread * -0.5f;
            SnowballImpactPuff.AddVelocity = SnowballImpactPuffSpread;

            World.SpawnParticles(SnowballImpactPuff);
            Die();
        }


        public virtual void SetRotation()
        {
            EntityPos pos = (World is IServerWorldAccessor) ? ServerPos : Pos;

            double speed = pos.Motion.Length();

            if (CollisionBox == null) return;
            CollisionBox.X1 = -0.2f;
            CollisionBox.X2 = 0.2f;
            CollisionBox.Z1 = -0.2f;
            CollisionBox.Z2 = 0.2f;
            CollisionBox.Y1 = 0f;
            CollisionBox.Y2 = 0.2f;
        }
        public override bool CanCollect(Entity byEntity)
        {
            return false;
        }
        public override void OnCollideWithLiquid()
        {
            base.OnCollideWithLiquid();
            BreakSnowball();
        }

        public override void ToBytes(BinaryWriter writer, bool forClient)
        {
            base.ToBytes(writer, forClient);
            writer.Write(beforeCollided);
            ProjectileStack.ToBytes(writer);
        }

        public override void FromBytes(BinaryReader reader, bool fromServer)
        {
            base.FromBytes(reader, fromServer);
            beforeCollided = reader.ReadBoolean();
            
            ProjectileStack = World == null ? new ItemStack(reader) : new ItemStack(reader, World);
        }
    }
}
