﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Vintagestory.API.Client;
using Vintagestory.API.Common;
using Vintagestory.API.Common.Entities;
using Vintagestory.API.MathTools;
using Vintagestory.API.Util;
using Vintagestory.GameContent;

//Note from Playah: Some of the following code has been copied and haphazardly modified from
//https://github.com/anegostudios/vssurvivalmod/blob/6a614481136cba45aad647eed5a8dac29a3c847e/Block/BlockSnow.cs (1.15-pre.1)

namespace snowballs
{
    public class BlockSnow_snowballs : Block
    {

        WorldInteraction[] interactions;
        public SimpleParticleProperties SnowLayerBreak = new SimpleParticleProperties();
        public Vec3f SnowLayerBreakSpread = new Vec3f();

        public override void OnLoaded(ICoreAPI api)
        {
            base.OnLoaded(api);

            //Default particle values
            SnowLayerBreak.MinQuantity = 15;
            SnowLayerBreak.Color = ColorUtil.ColorFromRgba(255, 245, 235, 255);
            SnowLayerBreak.AddPos = new Vec3d(1, 0.1, 1);
            SnowLayerBreak.LifeLength = 2f;
            SnowLayerBreak.addLifeLength = 1f;
            SnowLayerBreak.MinSize = 0.5f;
            SnowLayerBreak.MaxSize = 1.5f;
            SnowLayerBreak.SizeEvolve = new EvolvingNatFloat(EnumTransformFunction.LINEAR, -0.5f);
            SnowLayerBreak.GravityEffect = 1f;
            SnowLayerBreak.ParticleModel = EnumParticleModel.Cube;
            SnowLayerBreak.MinVelocity = new Vec3f(-0.5f, 1f, -0.5f);
            SnowLayerBreak.AddVelocity = new Vec3f(1f, 2f, 1f);

            interactions = ObjectCacheUtil.GetOrCreate(api, "snowblockBlockInteractions", () =>
            {
                return new WorldInteraction[] {
                    new WorldInteraction()
                    {
                        ActionLangCode = "blockhelp-snow-makesnowball",
                        HotKeyCode = "sneak",
                        MouseButton = EnumMouseButton.Right,
                        RequireFreeHand = true,
                        ShouldApply = (wi, bs, es) =>
                        {
                            BlockSnow_snowballs betm = api.World.BlockAccessor.GetBlock(bs.Position) as BlockSnow_snowballs;
                            bool TopIsFree = (api.World.BlockAccessor.GetBlock(bs.Position.UpCopy(1)).Replaceable > 9000); //Check if there's a block above the snowblock
                            return (betm != null & TopIsFree == true);
                        }
                    }
                };
            });
        }

        Cuboidf[] fullBox = new Cuboidf[] { new Cuboidf(0, 0, 0, 1, 1, 1) };
        public override Cuboidf[] GetCollisionBoxes(IBlockAccessor world, BlockPos pos)
        {
            if (world.GetBlock(pos.X, pos.Y + 1, pos.Z) is BlockLayered)
            {
                return fullBox;
            }

            return base.GetCollisionBoxes(world, pos);
        }

        /// <summary>
        /// No need to render the face of lake / glacier ice adjoining solid snow blocks; and this will help with z-fighting in arctic landscapes
        /// </summary>
        public override bool ShouldMergeFace(int facingIndex, Block neighbourIce, int intraChunkIndex3d)
        {
            return true;
        }

        public override bool OnBlockInteractStart(IWorldAccessor world, IPlayer byPlayer, BlockSelection blockSel)
        {
            BlockPos pos = blockSel.Position;
            if (byPlayer.Entity.Controls.Sneak & byPlayer.Entity.RightHandItemSlot.Empty & world.BlockAccessor.GetBlock(blockSel.Position.UpCopy(1)).Replaceable > 9000)
            {
                ItemStack snowball = new ItemStack(world.GetItem(new AssetLocation("snowballs:snowball")));
                byPlayer.Entity.RightHandItemSlot.Itemstack = snowball;
                world.PlaySoundAt(new AssetLocation("sounds/block/snow"), byPlayer, byPlayer);

                SnowLayerBreak.MinPos = pos.ToVec3d() + new Vec3d(0, 0.875, 0);
                world.SpawnParticles(SnowLayerBreak);

                Block block = world.GetBlock(new AssetLocation("snowlayer-7"));
                world.BlockAccessor.SetBlock(block.Id, pos);
                return true;
            }
            return false;
        }

        public override WorldInteraction[] GetPlacedBlockInteractionHelp(IWorldAccessor world, BlockSelection selection, IPlayer forPlayer)
        {
            return interactions.Append(base.GetPlacedBlockInteractionHelp(world, selection, forPlayer));
        }
    }
}