﻿using System;
using Vintagestory.API.Client;
using Vintagestory.API.Common;
using Vintagestory.API.MathTools;
using Vintagestory.API.Util;
using Vintagestory.GameContent;

//Note from Playah: Some of the following code has been copied and haphazardly modified from
//https://github.com/anegostudios/vssurvivalmod/blob/c03da7a3f54a93bd11765e5b4c32c1a67a989e3f/Block/BlockSnowLayer.cs (v1.14)

namespace snowballs
{
    public class SnowBlockLayer_snowballs : BlockLayered
    {

        int height;
        bool canMelt;
        WorldInteraction[] interactions;

        public SimpleParticleProperties SnowLayerBreak = new SimpleParticleProperties();
        public Vec3f SnowLayerBreakSpread = new Vec3f();

        public override void OnLoaded(ICoreAPI api)
        {
            base.OnLoaded(api);

            height = Variant["height"].ToInt();
            snowLevel = height;

            notSnowCovered = api.World.GetBlock(0);
            snowCovered1 = api.World.GetBlock(CodeWithVariant("height", "1"));
            snowCovered2 = api.World.GetBlock(CodeWithVariant("height", "2"));
            snowCovered3 = api.World.GetBlock(CodeWithVariant("height", "3"));

            canMelt = api.World.Config.GetBool("snowAccum", true);

            //Default particle values
            SnowLayerBreak.MinQuantity = 15;
            SnowLayerBreak.Color = ColorUtil.ColorFromRgba(255, 245, 235, 255);
            SnowLayerBreak.AddPos = new Vec3d(1, 0.1, 1);
            SnowLayerBreak.LifeLength = 2f;
            SnowLayerBreak.addLifeLength = 1f;
            SnowLayerBreak.MinSize = 0.5f;
            SnowLayerBreak.MaxSize = 1.5f;
            SnowLayerBreak.SizeEvolve = new EvolvingNatFloat(EnumTransformFunction.LINEAR, -0.5f);
            SnowLayerBreak.GravityEffect = 1f;
            SnowLayerBreak.ParticleModel = EnumParticleModel.Cube;
            SnowLayerBreak.MinVelocity = new Vec3f(-0.5f, 1f, -0.5f);
            SnowLayerBreak.AddVelocity = new Vec3f(1f, 2f, 1f);

            interactions = ObjectCacheUtil.GetOrCreate(api, "snowblocklayerBlockInteractions", () =>
            {
                return new WorldInteraction[] {
                    new WorldInteraction()
                    {
                        ActionLangCode = "blockhelp-snow-makesnowball",
                        HotKeyCode = "sneak",
                        MouseButton = EnumMouseButton.Right,
                        RequireFreeHand = true,
                        ShouldApply = (wi, bs, es) =>
                        {
                            SnowBlockLayer_snowballs betm = api.World.BlockAccessor.GetBlock(bs.Position) as SnowBlockLayer_snowballs;
                            return betm != null;
                        }
                    }
                };
            });
        }

        public override bool OnBlockInteractStart(IWorldAccessor world, IPlayer byPlayer, BlockSelection blockSel)
        {
            BlockPos pos = blockSel.Position;
            if (byPlayer.Entity.Controls.Sneak & byPlayer.Entity.RightHandItemSlot.Empty)
            {
                ItemStack snowball = new ItemStack(world.GetItem(new AssetLocation("snowballs:snowball")));
                byPlayer.Entity.RightHandItemSlot.Itemstack = snowball;
                world.PlaySoundAt(new AssetLocation("sounds/block/snow"), byPlayer, byPlayer);

                SnowLayerBreak.MinPos = pos.ToVec3d() + new Vec3d(0, (height * 0.125) - 0.125, 0);
                world.SpawnParticles(SnowLayerBreak);

                if (height == 1)
                {
                    world.BlockAccessor.SetBlock(0, pos);
                }
                else
                {
                    Block block = world.GetBlock(CodeWithVariant("height", "" + (height - 1)));
                    world.BlockAccessor.SetBlock(block.Id, pos);
                }
                return true;
            }
            return false;
        }

        public override WorldInteraction[] GetPlacedBlockInteractionHelp(IWorldAccessor world, BlockSelection selection, IPlayer forPlayer)
        {
            return interactions.Append(base.GetPlacedBlockInteractionHelp(world, selection, forPlayer));
        }

        public override bool ShouldReceiveServerGameTicks(IWorldAccessor world, BlockPos pos, Random offThreadRandom, out object extra)
        {
            extra = null;
            if (!canMelt) return false;
            ClimateCondition conds = world.BlockAccessor.GetClimateAt(pos, EnumGetClimateMode.NowValues);
            return conds != null && offThreadRandom.NextDouble() < GameMath.Clamp((conds.Temperature - 0.5f) / (15f - 10f * conds.Rainfall), 0, 1);
        }

        public void UpdateSnowLayer(IWorldAccessor world, BlockPos pos)
        {
            if (height == 1)
            {
                world.BlockAccessor.SetBlock(0, pos);
                return;
            }

            Block block = world.GetBlock(CodeWithVariant("height", "" + (height - 1)));
            world.BlockAccessor.SetBlock(block.Id, pos);
        }
        public override void OnServerGameTick(IWorldAccessor world, BlockPos pos, object extra = null)
        {
            UpdateSnowLayer(world, pos);
        }
    }
}
